@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2008 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.153 2008/01/16 16:24:49 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.160 2009/03/15 10:03:42 vapier Exp $
  *
  * Copyright 2005-2008 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2008 Mike Frysinger  - <vapier@gentoo.org>
@@ -31,6 +31,7 @@
 
 /* prototypes and such */
 static char eat_file(const char *file, char *buf, const size_t bufsize);
+int charmatch(const char *str1, const char *str2);
 int rematch(const char *, const char *, int);
 static char *rmspace(char *);
 
@@ -54,7 +55,7 @@ char reinitialize = 0;
 char reinitialize_metacache = 0;
 char portdir[_Q_PATH_MAX] = "/usr/portage";
 char portarch[20] = "";
-char portvdb[] = "var/db/pkg";
+char portvdb[_Q_PATH_MAX] = "var/db/pkg";
 char portcachedir[] = "metadata/cache";
 char portroot[_Q_PATH_MAX] = "/";
 char config_protect[_Q_PATH_MAX] = "/etc/";
@@ -202,15 +203,33 @@ close_and_ret:
 	return ret;
 }
 
-int rematch(const char *regex, const char *match, int cflags)
+/* if all chars in str1 coincide with the begining of the str2 return 0 */
+int charmatch(const char *str1, const char *str2)
+{
+	if ((str1 == NULL) || (str2 == NULL))
+		return EXIT_FAILURE;
+
+	while (*str1 != 0) {
+		/* if str2==0 but str1!=0 we'll return and so it's
+		 * impossible to touch match++ address here */
+		if (*str1 != *str2)
+			return 1;
+		str1++;
+		str2++;
+	}
+
+	return 0;
+}
+
+int rematch(const char *re, const char *match, int cflags)
 {
 	regex_t preg;
 	int ret;
 
-	if ((match == NULL) || (regex == NULL))
+	if ((match == NULL) || (re == NULL))
 		return EXIT_FAILURE;
 
-	if ((ret = regcomp(&preg, regex, cflags))) {
+	if ((ret = regcomp(&preg, re, cflags))) {
 		char err[256];
 		if (regerror(ret, &preg, err, sizeof(err)))
 			warnf("regcomp failed: %s", err);
@@ -449,6 +468,7 @@ void initialize_portage_env(void)
 {
 	char nocolor = 0;
 	int i, f;
+	ssize_t profilelen;
 	struct stat st;
 	FILE *fp;
 	char buf[BUFSIZE], *s, *p;
@@ -476,15 +496,25 @@ void initialize_portage_env(void)
 		{"ROOT",              4, _Q_STR,  portroot,       sizeof(portroot)}
 	};
 
+	s = getenv("Q_VDB");	/* #257251 */
+	if (s) {
+		strncpy(portvdb, s, sizeof(portvdb));
+		portvdb[sizeof(portvdb) - 1] = '\0';
+	}
+
 	if ((p = strchr(portroot, '/')) != NULL)
 		if (strlen(p) != 1)
 			strncat(portroot, "/", sizeof(portroot));
 
 	f = 0;
-	if (readlink("/etc/make.profile", profile, sizeof(profile)) == -1)
+	profilelen = readlink("/etc/make.profile", profile, sizeof(profile) - 1);
+	if (profilelen == -1)
 		strcpy(profile, "/etc/make.profile");
+	else
+		profile[profilelen]='\0';
+
 	if (profile[0] != '/') {
-		memmove(profile+5, profile, strlen(profile));
+		memmove(profile+5, profile, strlen(profile)+1);
 		memcpy(profile, "/etc/", 5);
 	}
 	do {
@@ -564,6 +594,10 @@ void initialize_portage_env(void)
 			}
 		}
 	}
+	if ((p = strchr(portroot, '/')) != NULL)
+		if (strlen(p) != 1)
+			strncat(portroot, "/", sizeof(portroot));
+
 	if (getenv("PORTAGE_QUIET") != NULL)
 		quiet = 1;
 
@@ -709,11 +743,7 @@ const char *initialize_flat(int cache_type)
 
 	warn("Finished %u entries in %d.%06d seconds", count, secs, frac);
 	if (secs > 120)
-#ifdef __linux__
-		warn("You should consider a faster file system such as reiserfs for PORTDIR='%s'", portdir);
-#else
 		warn("You should consider using the noatime mount option for PORTDIR='%s' if it's not already enabled", portdir);
-#endif
 ret:
 	return cache_file;
 }
@@ -755,6 +785,7 @@ typedef struct {
 	char *PDEPEND;
 	char *PROVIDE;       /* line 14 */
 	char *EAPI;
+	char *PROPERTIES;
 	depend_atom *atom;
 } portage_cache;
 
@@ -856,6 +887,7 @@ portage_cache *cache_read_file(const char *file)
 	next_line(CDEPEND, PDEPEND)
 	next_line(PDEPEND, PROVIDE)
 	next_line(PROVIDE, EAPI)
+	next_line(EAPI, PROPERTIES)
 #undef next_line
 	ptr = strchr(ptr+1, '\n');
 	*ptr = '\0';
@@ -890,6 +922,7 @@ void cache_dump(portage_cache *cache)
 	printf("PDEPEND    : %s\n", cache->PDEPEND);
 	printf("PROVIDE    : %s\n", cache->PROVIDE);
 	printf("EAPI       : %s\n", cache->EAPI);
+	printf("PROPERTIES : %s\n", cache->PROPERTIES);
 	if (!cache->atom) return;
 	printf("CATEGORY   : %s\n", cache->atom->CATEGORY);
 	printf("PN         : %s\n", cache->atom->PN);
